@@ -28,40 +28,36 @@ For news, bugfixes, etc. visit the home page for this implementation at
     http://www.wayforward.net/spf/
 """
 
-# CVS Commits since last release (2.0.2):
+# CVS Commits since last release (2.0.3):
 # $Log: spf.py,v $
-# Revision 1.108.2.13  2007/01/15 19:14:27  customdesigned
-# Permerror for more than one exp= or redirect=
+# Revision 1.108.2.22  2007/06/23 20:17:09  customdesigned
+# Don't try to include null (None) keyword values.
 #
-# Revision 1.108.2.12  2007/01/13 18:45:33  customdesigned
-# Record matching mechanism.
+# Revision 1.108.2.21  2007/03/29 19:38:03  customdesigned
+# Remove trailing ';' again, fix Received-SPF tests.
 #
-# Revision 1.108.2.11  2007/01/13 18:21:41  customdesigned
-# Test for RFC4408 6.2/4, and fix spf.py to comply.
+# Revision 1.108.2.20  2007/03/27 20:54:22  customdesigned
+# Correct Received-SPF header format.
 #
-# Revision 1.123  2007/01/11 18:49:37  customdesigned
-# Add mechanism to Received-SPF header.
+# Revision 1.108.2.19  2007/03/17 19:07:01  customdesigned
+# For default modifier, return ambiguous in harsh mode, ignore in strict mode,
+# follow in lax mode.
 #
-# Revision 1.122  2007/01/11 18:25:54  customdesigned
-# Record matching mechanism.
+# Revision 1.108.2.18  2007/03/17 18:25:38  customdesigned
+# Default modifier is obsolete.  Retab (expandtab) spf.py
 #
-# Revision 1.108.2.10  2007/01/13 00:46:35  kitterma
-# Update copyright statements for new year.
+# Revision 1.108.2.17  2007/03/13 20:13:16  customdesigned
+# Missing parentheses.
 #
-# Revision 1.108.2.9  2007/01/12 22:14:56  kitterma
-# Change DNS queries to only check Type SPF in Harsh mode
+# Revision 1.108.2.16  2007/01/25 20:50:13  kitterma
+# Update versions to reflect working on 2.0.4 now.
 #
-# Revision 1.108.2.8  2007/01/06 22:58:21  kitterma
-# Update changelogs and version to reflect 2.0.2 released and 2.0.3 started.
-#
-# Revision 1.108.2.7  2007/01/06 21:03:15  customdesigned
-# Tested spf.py in python2.2.
 #
 # See pyspf_changelog.txt for earlier CVS commits.
 
 __author__ = "Terence Way"
 __email__ = "terry@wayforward.net"
-__version__ = "2.0.3: January 12, 2007"
+__version__ = "2.0.4: January 24, 2007"
 MODULE = 'spf'
 
 USAGE = """To check an incoming mail request:
@@ -97,11 +93,11 @@ def DNSLookup(name, qtype, strict=True):
     try:
         req = DNS.DnsRequest(name, qtype=qtype)
         resp = req.req()
-	#resp.show()
+        #resp.show()
         # key k: ('wayforward.net', 'A'), value v
-	# FIXME: pydns returns AAAA RR as 16 byte binary string, but
-	# A RR as dotted quad.  For consistency, this driver should
-	# return both as binary string.
+        # FIXME: pydns returns AAAA RR as 16 byte binary string, but
+        # A RR as dotted quad.  For consistency, this driver should
+        # return both as binary string.
         return [((a['name'], a['typename']), a['data']) for a in resp.answers]
     except IOError, x:
         raise TempError, 'DNS ' + str(x)
@@ -128,7 +124,7 @@ RE_IP4 = re.compile(PAT_IP4+'$')
 
 RE_TOPLAB = re.compile(
     r'\.(?:[0-9a-z]*[a-z][0-9a-z]*|[0-9a-z]+-[0-9a-z-]*[0-9a-z])\.?$|%s'
-    	% PAT_CHAR, re.IGNORECASE)
+        % PAT_CHAR, re.IGNORECASE)
 
 RE_DOT_ATOM = re.compile(r'%(atext)s+([.]%(atext)s+)*$' % {
     'atext': r"[0-9a-z!#$%&'*+/=?^_`{}|~-]" }, re.IGNORECASE)
@@ -303,13 +299,13 @@ class query(object):
         self.s, self.h = s, h
         if not s and h:
             self.s = 'postmaster@' + h
-	    self.ident = 'helo'
-	else:
-	    self.ident = 'mailfrom'
+            self.ident = 'helo'
+        else:
+            self.ident = 'mailfrom'
         self.l, self.o = split_email(s, h)
         self.t = str(int(time.time()))
         self.d = self.o
-        self.p = None	# lazy evaluation
+        self.p = None   # lazy evaluation
         if receiver:
             self.r = receiver
         else:
@@ -324,35 +320,36 @@ class query(object):
         self.lookups = 0
         # strict can be False, True, or 2 (numeric) for harsh
         self.strict = strict
-	if i:
-	    self.set_ip(i)
+        if i:
+            self.set_ip(i)
+        self.default_modifier = True
 
     def set_ip(self, i):
         "Set connect ip, and ip6 or ip4 mode."
-	if RE_IP4.match(i):
-	    self.ip = addr2bin(i)
-	    ip6 = False
-	else:
-	    self.ip = bin2long6(inet_pton(i))
-	    if (self.ip >> 32) == 0xFFFF:	# IP4 mapped address
-		self.ip = self.ip & 0xFFFFFFFFL
-		ip6 = False
-	    else:
-		ip6 = True
-	# NOTE: self.A is not lowercase, so isn't a macro.  See query.expand()
-	if ip6:
-	    self.c = inet_ntop(
-	    	struct.pack("!QQ", self.ip>>64, self.ip&0xFFFFFFFFFFFFFFFFL))
-	    self.i = '.'.join(list('%032X'%self.ip))
-	    self.A = 'AAAA'
-	    self.v = 'ip6'
-	    self.cidrmax = 128
-	else:
-	    self.c = socket.inet_ntoa(struct.pack("!L", self.ip))
-	    self.i = self.c
-	    self.A = 'A'
-	    self.v = 'in-addr'
-	    self.cidrmax = 32
+        if RE_IP4.match(i):
+            self.ip = addr2bin(i)
+            ip6 = False
+        else:
+            self.ip = bin2long6(inet_pton(i))
+            if (self.ip >> 32) == 0xFFFF:       # IP4 mapped address
+                self.ip = self.ip & 0xFFFFFFFFL
+                ip6 = False
+            else:
+                ip6 = True
+        # NOTE: self.A is not lowercase, so isn't a macro.  See query.expand()
+        if ip6:
+            self.c = inet_ntop(
+                struct.pack("!QQ", self.ip>>64, self.ip&0xFFFFFFFFFFFFFFFFL))
+            self.i = '.'.join(list('%032X'%self.ip))
+            self.A = 'AAAA'
+            self.v = 'ip6'
+            self.cidrmax = 128
+        else:
+            self.c = socket.inet_ntoa(struct.pack("!L", self.ip))
+            self.i = self.c
+            self.A = 'A'
+            self.v = 'in-addr'
+            self.cidrmax = 32
 
     def set_default_explanation(self, exp):
         exps = self.exps
@@ -372,23 +369,28 @@ class query(object):
             p = self.validated_ptrs()
             if not p:
                 self.p = "unknown"
-	    elif self.d in p:
-	        self.p = self.d
-	    else:
-	        sfx = '.' + self.d
-	        for d in p:
-		    if d.endswith(sfx):
-		        self.p = d
-			break
-		else:
-		    self.p = p[0]
+            elif self.d in p:
+                self.p = self.d
+            else:
+                sfx = '.' + self.d
+                for d in p:
+                    if d.endswith(sfx):
+                        self.p = d
+                        break
+                else:
+                    self.p = p[0]
         return self.p
 
     def best_guess(self, spf=DEFAULT_SPF):
-        """Return a best guess based on a default SPF record"""
+        """Return a best guess based on a default SPF record.
+    >>> q = query('1.2.3.4','','SUPERVISION1',receiver='example.com')
+    >>> q.best_guess()[0]
+    'none'
+        """
+        if RE_TOPLAB.split(self.d)[-1]:
+            return ('none', 250, '')
         return self.check(spf)
 
-
     def check(self, spf=None):
         """
     Returns (result, mta-status-code, explanation) where result
@@ -456,8 +458,8 @@ class query(object):
         # will continue processing.  However, the exception
         # that strict processing would raise is saved here
         self.perm_error = None
-	self.mechanism = None
-	self.options = {}
+        self.mechanism = None
+        self.options = {}
 
         try:
             self.lookups = 0
@@ -467,12 +469,12 @@ class query(object):
                 spf = insert_libspf_local_policy(
                     spf, self.libspf_local)
             rc = self.check1(spf, self.d, 0)
-	    if self.perm_error:
-		# lax processing encountered a permerror, but continued
-		self.perm_error.ext = rc
-		raise self.perm_error
-	    return rc
-	        
+            if self.perm_error:
+                # lax processing encountered a permerror, but continued
+                self.perm_error.ext = rc
+                raise self.perm_error
+            return rc
+                
         except TempError, x:
             self.prob = x.msg
             if x.mech:
@@ -526,6 +528,13 @@ class query(object):
                 self.perm_error = x
         return self.perm_error
 
+    def expand_domain(self,arg):
+        "validate and expand domain-spec"
+        # any trailing dot was removed by expand()
+        if RE_TOPLAB.split(arg)[-1]:
+            raise PermError('Invalid domain found (use FQDN)', arg)
+        return self.expand(arg)
+
     def validate_mechanism(self, mech):
         """Parse and validate a mechanism.
     Returns mech,m,arg,cidrlength,result
@@ -583,7 +592,7 @@ class query(object):
     """
         if mech.endswith( "," ):
             self.note_error('Do not separate mechnisms with commas', mech)
-	    mech = mech[:-1]
+            mech = mech[:-1]
         # a mechanism
         m, arg, cidrlength, cidr6length = parse_mechanism(mech, self.d)
         # map '?' '+' or '-' to 'neutral' 'pass' or 'fail'
@@ -615,8 +624,8 @@ class query(object):
                 cidr6length = 128
             elif cidr6length > 128:
                 raise PermError('Invalid IP6 CIDR length', mech)
-	    if self.v == 'ip6':
-	    	cidrlength = cidr6length
+            if self.v == 'ip6':
+                cidrlength = cidr6length
         elif m == 'ip4':
             if cidr6length is not None:
                 raise PermError('Dual CIDR not allowed', mech)
@@ -638,14 +647,14 @@ class query(object):
         else:
             if cidrlength is not None or cidr6length is not None:
                 raise PermError('CIDR not allowed', mech)
-	    cidrlength = self.cidrmax
+            cidrlength = self.cidrmax
 
-        # validate domain-spec
         if m in ('a', 'mx', 'ptr', 'exists', 'include'):
-            # any trailing dot was removed by expand()
-            if RE_TOPLAB.split(arg)[-1]:
-                raise PermError('Invalid domain found (use FQDN)', arg)
-            arg = self.expand(arg)
+            if m == 'exists' and not arg:
+                raise PermError('implicit exists not allowed', mech)
+            arg = self.expand_domain(arg)
+            if not arg:
+                raise PermError('empty domain:',mech)
             if m == 'include':
                 if arg == self.d:
                     if mech != 'include':
@@ -686,8 +695,8 @@ class query(object):
         # for common mistakes like IN TXT "v=spf1" "mx" "-all"
         # in relaxed mode.
         if spf[0].lower() != 'v=spf1':
-	    assert strict > 1
-	    raise AmbiguityWarning('Invalid SPF record in', self.d)
+            assert strict > 1
+            raise AmbiguityWarning('Invalid SPF record in', self.d)
         spf = spf[1:]
 
         # copy of explanations to be modified by exp=
@@ -700,7 +709,7 @@ class query(object):
         default = 'neutral'
         mechs = []
 
-	modifiers = []
+        modifiers = []
         # Look for modifiers
         #
         for mech in spf:
@@ -709,34 +718,41 @@ class query(object):
                 mechs.append(self.validate_mechanism(mech))
                 continue
 
-	    mod,arg = m
-	    if mod in modifiers:
-	    	if mod == 'redirect':
-		    raise PermError('redirect= MUST appear at most once',mech)
-		self.note_error('%s= MUST appear at most once'%mod,mech)
-		# just use last one in lax mode
-	    modifiers.append(mod)
-            if m[0] == 'exp':
-	        # always fetch explanation to check permerrors
-	        exp = self.get_explanation(m[1])
-	        if exp and not recursion:
-		    # only set explanation in base recursion level
-		    self.set_explanation(exp)
-            elif m[0] == 'redirect':
+            mod,arg = m
+            if mod in modifiers:
+                if mod == 'redirect':
+                    raise PermError('redirect= MUST appear at most once',mech)
+                self.note_error('%s= MUST appear at most once'%mod,mech)
+                # just use last one in lax mode
+            modifiers.append(mod)
+            if mod == 'exp':
+                # always fetch explanation to check permerrors
+                arg = self.expand_domain(arg)
+                exp = self.get_explanation(arg)
+                if exp and not recursion:
+                    # only set explanation in base recursion level
+                    self.set_explanation(exp)
+            elif mod == 'redirect':
                 self.check_lookups()
-                redirect = self.expand(m[1])
-            elif m[0] == 'default':
-		arg = self.expand(m[1])
-                # default=- is the same as default=fail
-                default = RESULTS.get(arg, default)
-	    elif mod == 'op':
-	    	if not recursion:
-		    for v in arg.split('.'):
-		        if v: self.options[v] = True
-	    else:
-		# spf rfc: 3.6 Unrecognized Mechanisms and Modifiers
-		self.expand(m[1])	# syntax error on invalid macro
-
+                redirect = self.expand_domain(arg)
+                if not redirect:
+                    raise PermError('redirect has empty domain:',arg)
+            elif mod == 'default':
+                # default modifier is obsolete
+                if self.strict > 1:
+                    raise AmbiguityWarning('The default= modifier is obsolete.')
+                if not self.strict and self.default_modifier:
+                    # might be an old policy, so do it anyway
+                    arg = self.expand(arg)
+                    # default=- is the same as default=fail
+                    default = RESULTS.get(arg, default)
+            elif mod == 'op':
+                if not recursion:
+                    for v in arg.split('.'):
+                        if v: self.options[v] = True
+            else:
+                # spf rfc: 3.6 Unrecognized Mechanisms and Modifiers
+                self.expand(m[1])       # syntax error on invalid macro
 
         # Evaluate mechanisms
         #
@@ -769,8 +785,8 @@ class query(object):
 
             elif m == 'a':
                 self.check_lookups()
-		if self.cidrmatch(self.dns_a(arg,self.A), cidrlength):
-		    break
+                if self.cidrmatch(self.dns_a(arg,self.A), cidrlength):
+                    break
 
             elif m == 'mx':
                 self.check_lookups()
@@ -778,19 +794,19 @@ class query(object):
                     break
 
             elif m == 'ip4':
-	        if self.v == 'in-addr': # match own connection type only
-		    try:
-			if self.cidrmatch([arg], cidrlength): break
-		    except socket.error:
-			raise PermError('syntax error', mech)
+                if self.v == 'in-addr': # match own connection type only
+                    try:
+                        if self.cidrmatch([arg], cidrlength): break
+                    except socket.error:
+                        raise PermError('syntax error', mech)
 
             elif m == 'ip6':
-	        if self.v == 'ip6': # match own connection type only
-		    try:
-			arg = inet_pton(arg)
-			if self.cidrmatch([arg], cidrlength): break
-		    except socket.error:
-			raise PermError('syntax error', mech)
+                if self.v == 'ip6': # match own connection type only
+                    try:
+                        arg = inet_pton(arg)
+                        if self.cidrmatch([arg], cidrlength): break
+                    except socket.error:
+                        raise PermError('syntax error', mech)
 
             elif m == 'ptr':
                 self.check_lookups()
@@ -805,16 +821,16 @@ class query(object):
                 if not redirect_record:
                     raise PermError('redirect domain has no SPF record',
                         redirect)
-		# forget modifiers on redirect
-		if not recursion:
-		  self.exps = dict(self.defexps)
-		  self.options = {}
+                # forget modifiers on redirect
+                if not recursion:
+                  self.exps = dict(self.defexps)
+                  self.options = {}
                 return self.check1(redirect_record, redirect, recursion)
-	    result = default
-	    mech = None
+            result = default
+            mech = None
 
-	if not recursion:	# record matching mechanism at base level
-	    self.mechanism = mech
+        if not recursion:       # record matching mechanism at base level
+            self.mechanism = mech
         if result == 'fail':
             return (result, 550, exps[result])
         else:
@@ -823,21 +839,25 @@ class query(object):
     def check_lookups(self):
         self.lookups = self.lookups + 1
         if self.lookups > MAX_LOOKUP*4:
-            raise PermError('More than %d DNS lookups'%MAX_LOOKUP*4)
+            raise PermError('More than %d DNS lookups'%(MAX_LOOKUP*4))
         if self.lookups > MAX_LOOKUP:
             self.note_error('Too many DNS lookups')
 
     def get_explanation(self, spec):
         """Expand an explanation."""
         if spec:
-            a = self.dns_txt(self.expand(spec))
-	    if len(a) == 1:
-	    	try:
-		    return self.expand(a[0], stripdot=False)
-		except PermError:
-		    # RFC4408 6.2/4 syntax errors cause exp= to be ignored
-		    pass
-	return None
+            a = self.dns_txt(spec)
+            if len(a) == 1:
+                try:
+                    return self.expand(a[0], stripdot=False)
+                except PermError:
+                    # RFC4408 6.2/4 syntax errors cause exp= to be ignored
+                    pass
+        if self.strict > 1:
+            raise PermError('Empty domain-spec on exp=')
+        # RFC4408 6.2/4 empty domain spec is ignored
+        # (unless you give precedence to the grammar).
+        return None
 
     def expand(self, str, stripdot=True): # macros='slodipvh'
         """Do SPF RFC macro expansion.
@@ -955,16 +975,16 @@ class query(object):
 #                print letter
                 if letter == 'p':
                     self.getp()
-		elif letter in 'crt' and stripdot:
-		    raise PermError(
-		        'c,r,t macros allowed in exp= text only', macro)
+                elif letter in 'crt' and stripdot:
+                    raise PermError(
+                        'c,r,t macros allowed in exp= text only', macro)
                 expansion = getattr(self, letter, self)
                 if expansion:
                     if expansion == self:
                         raise PermError('Unknown Macro Encountered', macro) 
-		    e = expand_one(expansion, macro[3:-1], JOINERS.get(letter))
-		    if letter != macro[2]:
-		        e = urllib.quote(e)
+                    e = expand_one(expansion, macro[3:-1], JOINERS.get(letter))
+                    if letter != macro[2]:
+                        e = urllib.quote(e)
                     result += e
 
             end = i.end()
@@ -981,11 +1001,11 @@ class query(object):
         name.  Returns None if not found, or if more than one record
         is found.
         """
-	# Per RFC 4.3/1, check for malformed domain.  This produces
-	# no results as a special case.
-	for label in domain.split('.'):
-	  if not label or len(label) > 63:
-	    return None
+        # Per RFC 4.3/1, check for malformed domain.  This produces
+        # no results as a special case.
+        for label in domain.split('.'):
+          if not label or len(label) > 63:
+            return None
         # for performance, check for most common case of TXT first
         a = [t for t in self.dns_txt(domain) if RE_SPF.match(t)]
         if len(a) > 1:
@@ -1052,13 +1072,13 @@ class query(object):
 
     def dns_a(self, domainname, A='A'):
         """Get a list of IP addresses for a domainname.
-	"""
+        """
         if not domainname: return []
         if self.strict > 1:
             alist = self.dns(domainname, A)
             if len(alist) == 0:
                 raise AmbiguityWarning(
-			'No %s records found for'%A, domainname)
+                        'No %s records found for'%A, domainname)
             else:
                 return alist
         return self.dns(domainname, A)
@@ -1074,19 +1094,19 @@ class query(object):
                     ptrnames = self.dns_ptr(self.i)
                     if len(ptrnames) > max:
                         warning = 'More than %d PTR records returned' % max
-                        raise AmbiguityWarning(warning, i)
+                        raise AmbiguityWarning(warning, self.i)
                     else:
                         if len(ptrnames) == 0:
                             raise AmbiguityWarning(
                                 'No PTR records found for ptr mechanism', self.c)
                 except:
                     raise AmbiguityWarning(
-                      'No PTR records found for ptr mechanism', i)
+                      'No PTR records found for ptr mechanism', self.i)
         else:
             max = MAX_PTR * 4
-	cidrlength = self.cidrmax
+        cidrlength = self.cidrmax
         return [p for p in self.dns_ptr(self.i)[:max]
-	    if self.cidrmatch(self.dns_a(p,self.A),cidrlength)]
+            if self.cidrmatch(self.dns_a(p,self.A),cidrlength)]
 
     def dns_ptr(self, i):
         """Get a list of domain names for an IP address."""
@@ -1126,12 +1146,12 @@ class query(object):
         cname = None
 
         if not result:
-	    safe2cache = query.SAFE2CACHE
+            safe2cache = query.SAFE2CACHE
             for k, v in DNSLookup(name, qtype, self.strict):
                 if k == (name, 'CNAME'):
                     cname = v
-		if (qtype,k[1]) in safe2cache:
-		    self.cache.setdefault(k, []).append(v)
+                if (qtype,k[1]) in safe2cache:
+                    self.cache.setdefault(k, []).append(v)
             result = self.cache.get( (name, qtype), [])
         if not result and cname:
             if not cnames:
@@ -1146,43 +1166,49 @@ class query(object):
         return result
 
     def cidrmatch(self, ipaddrs, n):
-	"""Match connect IP against a list of other IP addresses."""
-	try:
-	    if self.v == 'ip6':
-	        MASK = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL
-		bin = bin2long6
-	    else:
-	        MASK = 0xFFFFFFFFL
-		bin = addr2bin
-	    c = ~(MASK >> n) & MASK & self.ip
-	    for ip in [bin(ip) for ip in ipaddrs]:
-		if c == ~(MASK >> n) & MASK & ip: return True
-	except socket.error: pass
-	return False
+        """Match connect IP against a list of other IP addresses."""
+        try:
+            if self.v == 'ip6':
+                MASK = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFL
+                bin = bin2long6
+            else:
+                MASK = 0xFFFFFFFFL
+                bin = addr2bin
+            c = ~(MASK >> n) & MASK & self.ip
+            for ip in [bin(ip) for ip in ipaddrs]:
+                if c == ~(MASK >> n) & MASK & ip: return True
+        except socket.error: pass
+        return False
 
-    def get_header(self, res, receiver=None):
+    def get_header(self, res, receiver=None, **kv):
         if not receiver:
             receiver = self.r
-	client_ip = self.c
-	helo = quote_value(self.h)
-	if self.ident == 'helo':
-	    envelope_from = None
-	else:
-	    envelope_from = quote_value(self.s)
+        client_ip = self.c
+        helo = quote_value(self.h)
+        resmap = { 'pass': 'Pass', 'neutral': 'Neutral', 'fail': 'Fail',
+                'softfail': 'SoftFail', 'none': 'None',
+                'temperror': 'TempError', 'permerror': 'PermError' }
+        identity = self.ident
+        if identity == 'helo':
+            envelope_from = None
+        else:
+            envelope_from = quote_value(self.s)
+        tag = resmap[res]
         if res == 'permerror' and self.mech:
-            tag = ' '.join([res] + self.mech)
-	    problem = quote_value(' '.join(self.mech))
-	else:
-	    tag = res
-	    problem = None
-	mechanism = quote_value(self.mechanism)
-	res = ['%s (%s: %s)' % (tag,receiver,self.get_header_comment(res))]
-	for k in ('client_ip','envelope_from','helo','receiver',
-	  'problem','mechanism'):
-	    v = locals()[k]
-	    if v: res.append('%s=%s;'%(k,v))
-	res.append('identity=%s'%self.ident)
-	return ' '.join(res)
+            problem = quote_value(' '.join(self.mech))
+        else:
+            problem = None
+        mechanism = quote_value(self.mechanism)
+        res = ['%s (%s: %s)' % (tag,receiver,self.get_header_comment(res))]
+        for k in ('client_ip','envelope_from','helo','receiver',
+          'problem','mechanism'):
+            v = locals()[k]
+            if v: res.append('%s=%s;'%(k.replace('_','-'),v))
+        for k,v in kv.items():
+            if v: res.append('x-%s=%s;'%(k.replace('_','-'),quote_value(v)))
+        # do identity last so we can easily drop the trailing ';'
+        res.append('%s=%s'%('identity',identity))
+        return ' '.join(res)
 
     def get_header_comment(self, res):
         """Return comment for Received-SPF header.
@@ -1278,6 +1304,9 @@ def parse_mechanism(str, d):
     >>> parse_mechanism('a', 'foo.com')
     ('a', 'foo.com', None, None)
 
+    >>> parse_mechanism('exists','foo.com')
+    ('exists', None, None, None)
+
     >>> parse_mechanism('a:bar.com', 'foo.com')
     ('a', 'bar.com', None, None)
 
@@ -1313,7 +1342,9 @@ def parse_mechanism(str, d):
 
     a = str.split(':', 1)
     if len(a) < 2:
-        return str.lower(), d, cidr, cidr6
+        str = str.lower()
+        if str == 'exists': d = None
+        return str, d, cidr, cidr6
     return a[0].lower(), a[1], cidr, cidr6
 
 def reverse_dots(name):
@@ -1410,24 +1441,24 @@ else:
       """
       # convert to 8 words
       a = struct.unpack("!HHHHHHHH",s)
-      n = (0,0,0,0,0,0,0,0)	# null ip6
+      n = (0,0,0,0,0,0,0,0)     # null ip6
       if a == n: return '::'
       # check for ip4 mapped
       if a[:5] == (0,0,0,0,0) and a[5] in (0,0xFFFF):
-	ip4 = '.'.join([str(i) for i in struct.unpack("!HHHHHHBBBB",s)[6:]])
-	if a[5]:
-	  return "::FFFF:" + ip4
-	return "::" + ip4
+        ip4 = '.'.join([str(i) for i in struct.unpack("!HHHHHHBBBB",s)[6:]])
+        if a[5]:
+          return "::FFFF:" + ip4
+        return "::" + ip4
       # find index of longest sequence of 0
       for l in (7,6,5,4,3,2,1):
-	e = n[:l]
-	for i in range(9-l):
-	  if a[i:i+l] == e:
-	    if i == 0:
-	      return ':'+':%x'*(8-l) % a[l:]
-	    if i == 8 - l:
-	      return '%x:'*(8-l) % a[:-l] + ':'
-	    return '%x:'*i % a[:i] + ':%x'*(8-l-i) % a[i+l:]
+        e = n[:l]
+        for i in range(9-l):
+          if a[i:i+l] == e:
+            if i == 0:
+              return ':'+':%x'*(8-l) % a[l:]
+            if i == 8 - l:
+              return '%x:'*(8-l) % a[:-l] + ':'
+            return '%x:'*i % a[:i] + ':%x'*(8-l-i) % a[i+l:]
       return "%x:%x:%x:%x:%x:%x:%x:%x" % a
 
     def inet_pton(p):
@@ -1450,35 +1481,35 @@ else:
       ::1.2.3.4.5
       """
       if p == '::':
-	return '\0'*16
+        return '\0'*16
       s = p
       m = RE_IP4.search(s)
       try:
-	  if m:
-	      pos = m.start()
-	      ip4 = [int(i) for i in s[pos:].split('.')]
-	      if not pos:
-	          return struct.pack('!QLBBBB',0,65535,*ip4)
-	      s = s[:pos]+'%x%02x:%x%02x'%tuple(ip4)
-	  a = s.split('::')
-	  if len(a) == 2:
-	    l,r = a
-	    if not l:
-	      r = r.split(':')
-	      return struct.pack('!HHHHHHHH',
-		*[0]*(8-len(r)) + [int(s,16) for s in r])
-	    if not r:
-	      l = l.split(':')
-	      return struct.pack('!HHHHHHHH',
-		*[int(s,16) for s in l] + [0]*(8-len(l)))
-	    l = l.split(':')
-	    r = r.split(':')
-	    return struct.pack('!HHHHHHHH',
-		*[int(s,16) for s in l] + [0]*(8-len(l)-len(r))
-		+ [int(s,16) for s in r])
-	  if len(a) == 1:
-	    return struct.pack('!HHHHHHHH',
-		*[int(s,16) for s in a[0].split(':')])
+          if m:
+              pos = m.start()
+              ip4 = [int(i) for i in s[pos:].split('.')]
+              if not pos:
+                  return struct.pack('!QLBBBB',0,65535,*ip4)
+              s = s[:pos]+'%x%02x:%x%02x'%tuple(ip4)
+          a = s.split('::')
+          if len(a) == 2:
+            l,r = a
+            if not l:
+              r = r.split(':')
+              return struct.pack('!HHHHHHHH',
+                *[0]*(8-len(r)) + [int(s,16) for s in r])
+            if not r:
+              l = l.split(':')
+              return struct.pack('!HHHHHHHH',
+                *[int(s,16) for s in l] + [0]*(8-len(l)))
+            l = l.split(':')
+            r = r.split(':')
+            return struct.pack('!HHHHHHHH',
+                *[int(s,16) for s in l] + [0]*(8-len(l)-len(r))
+                + [int(s,16) for s in r])
+          if len(a) == 1:
+            return struct.pack('!HHHHHHHH',
+                *[int(s,16) for s in a[0].split(':')])
       except ValueError: pass
       raise ValueError,p
 
