@@ -66,6 +66,7 @@ class SPFTest(object):
     self.explanation = None
     self.spec = None
     self.header = None
+    self.strict = True
     self.receiver = None
     self.comment = []
     if 'result' not in data:
@@ -143,7 +144,7 @@ class SPFTestCase(unittest.TestCase):
     passed,failed = 0,0
     for t in tests:
       zonedata = t.scenario.zonedata
-      q = spf.query(i=t.host, s=t.mailfrom, h=t.helo)
+      q = spf.query(i=t.host, s=t.mailfrom, h=t.helo, strict=t.strict)
       q.set_default_explanation('DEFAULT')
       res,code,exp = q.check()
       if res in oldresults:
